import gmsh
import sys

gmsh.initialize(sys.argv)
gmsh.option.setNumber("General.Terminal", 1)

tri1 = [0., 1., 1.,
        0., 0., 1.,
        0., 0., 0.];
tri2 = [0., 1., 0.,
        0., 1., 1.,
        0., 0., 0.];

for step in range(0, 10):
    tri1.append(10.); tri1.append(10.); tri1.append(12. + step)
    tri2.append(10.); tri2.append(12. + step); tri2.append(13. + step)

t = gmsh.view.add("some data")

gmsh.view.addListData(t, "ST", 2, tri1 + tri2)

gmsh.view.write(t, "data.pos")

gmsh.finalize()



# This file reimplements gmsh/tutorial/t4.geo in Python.

import gmsh
import math

model = gmsh.model
factory = model.geo

gmsh.initialize()
gmsh.option.setNumber("General.Terminal", 1)

model.add("t4")

cm = 1e-02
e1 = 4.5 * cm; e2 = 6 * cm / 2; e3 =  5 * cm / 2
h1 = 5 * cm; h2 = 10 * cm; h3 = 5 * cm; h4 = 2 * cm; h5 = 4.5 * cm
R1 = 1 * cm; R2 = 1.5 * cm; r = 1 * cm
Lc1 = 0.01
Lc2 = 0.003

def hypot(a, b):
    return math.sqrt(a * a + b * b)

ccos = (-h5*R1 + e2 * hypot(h5, hypot(e2, R1))) / (h5*h5 + e2*e2)
ssin = math.sqrt(1 - ccos*ccos)

factory.addPoint(-e1-e2, 0    , 0, Lc1, 1)
factory.addPoint(-e1-e2, h1   , 0, Lc1, 2)
factory.addPoint(-e3-r , h1   , 0, Lc2, 3)
factory.addPoint(-e3-r , h1+r , 0, Lc2, 4)
factory.addPoint(-e3   , h1+r , 0, Lc2, 5)
factory.addPoint(-e3   , h1+h2, 0, Lc1, 6)
factory.addPoint( e3   , h1+h2, 0, Lc1, 7)
factory.addPoint( e3   , h1+r , 0, Lc2, 8)
factory.addPoint( e3+r , h1+r , 0, Lc2, 9)
factory.addPoint( e3+r , h1   , 0, Lc2, 10)
factory.addPoint( e1+e2, h1   , 0, Lc1, 11)
factory.addPoint( e1+e2, 0    , 0, Lc1, 12)
factory.addPoint( e2   , 0    , 0, Lc1, 13)

factory.addPoint( R1 / ssin, h5+R1*ccos, 0, Lc2, 14)
factory.addPoint( 0        , h5        , 0, Lc2, 15)
factory.addPoint(-R1 / ssin, h5+R1*ccos, 0, Lc2, 16)
factory.addPoint(-e2       , 0.0       , 0, Lc1, 17)

factory.addPoint(-R2 , h1+h3   , 0, Lc2, 18)
factory.addPoint(-R2 , h1+h3+h4, 0, Lc2, 19)
factory.addPoint( 0  , h1+h3+h4, 0, Lc2, 20)
factory.addPoint( R2 , h1+h3+h4, 0, Lc2, 21)
factory.addPoint( R2 , h1+h3   , 0, Lc2, 22)
factory.addPoint( 0  , h1+h3   , 0, Lc2, 23)
                                                
factory.addPoint( 0, h1+h3+h4+R2, 0, Lc2, 24)
factory.addPoint( 0, h1+h3-R2,    0, Lc2, 25)

factory.addLine(1 , 17, 1)
factory.addLine(17, 16, 2)

factory.addCircleArc(14,15,16, 3)
factory.addLine(14,13, 4)
factory.addLine(13,12, 5)
factory.addLine(12,11, 6)
factory.addLine(11,10, 7)
factory.addCircleArc(8,9,10, 8)
factory.addLine(8,7, 9)
factory.addLine(7,6, 10)
factory.addLine(6,5, 11)
factory.addCircleArc(3,4,5, 12)
factory.addLine(3,2, 13)
factory.addLine(2,1, 14)
factory.addLine(18,19, 15)
factory.addCircleArc(21,20,24, 16)
factory.addCircleArc(24,20,19, 17)
factory.addCircleArc(18,23,25, 18)
factory.addCircleArc(25,23,22, 19)
factory.addLine(21,22, 20)

factory.addCurveLoop([17,-15,18,19,-20,16], 21)
factory.addPlaneSurface([21], 22)
factory.addCurveLoop([11,-12,13,14,1,2,-3,4,5,6,7,-8,9,10], 23)

# A surface with one hole is specified using 2 curve loops:
factory.addPlaneSurface([23,21], 24)

# FIXME: this will be implemented through the gmshView API
#  View "comments" {
#    T2(10, -10, 0){ StrCat("Created on ", Today, " with Gmsh") };
#    T3(0, 0.11, 0, TextAttributes("Align", "Center", "Font", "Helvetica")){ "Hole" };
#    T3(0, 0.09, 0, TextAttributes("Align", "Center")){ "file://image.png@0.01x0" };
#    T3(-0.01, 0.09, 0, 0){ "file://image.png@0.01x0,0,0,1,0,1,0" };
#    T3(0, 0.12, 0, TextAttributes("Align", "Center")){ "file://image.png@0.01x0#" };
#    T2(350, -7, 0){ "file://image.png@20x0" };
# };

factory.synchronize()

model.mesh.generate(2)

gmsh.write("t4.msh")

gmsh.finalize()



import gmsh
import sys

gmsh.initialize(sys.argv)
gmsh.model.add("normals")
gmsh.model.occ.addSphere(0, 0, 0, 1)
gmsh.model.occ.addBox(2, 0, 0, 1, 1, 1)
gmsh.model.occ.synchronize()
gmsh.model.mesh.generate(2)

nn = []
cc = []

# get all surfaces
ent = gmsh.model.getEntities(2)

for e in ent:
    surf = e[1]
    # get nodes on surf, including those on the boundary (contrary to internal
    # nodes, which store their parametric coordinates, boundary nodes will be
    # reparametrized on surf in order to compute their parametric coordinates -
    # which will be different when reparametrized on another adjacent surface)
    tags, coord, param = gmsh.model.mesh.getNodes(2, surf, True)
    # get surface normal on all nodes, i.e. including on the geometrical
    # singularities (edges/points)
    normals = gmsh.model.getNormal(surf, param)
    # get surface curvature
    curv = gmsh.model.getCurvature(2, surf, param)
    for i in range(0,len(coord),3):
        nn.append(coord[i])
        nn.append(coord[i+1])
        nn.append(coord[i+2])
        nn.append(normals[i])
        nn.append(normals[i+1])
        nn.append(normals[i+2])
        cc.append(coord[i])
        cc.append(coord[i+1])
        cc.append(coord[i+2])
        cc.append(curv[i/3])

t = gmsh.view.add("normals")
gmsh.view.addListData(t, "VP", len(nn)/6, nn)
gmsh.view.write(t, "normals.pos")

t = gmsh.view.add("curvatures")
gmsh.view.addListData(t, "SP", len(cc)/4, cc)
gmsh.view.write(t, "curvatures.pos")

gmsh.finalize()



# This file reimplements gmsh/tutorial/t14.geo in Python.

# /*********************************************************************
#  *
#  *  Gmsh tutorial 14
#  *
#  *  Homology and cohomology computation
#  *
#  *********************************************************************/

# Homology computation in Gmsh finds representative chains of (relative)
# (co)homology space bases using a mesh of a model.  The representative basis
# chains are stored in the mesh as physical groups of Gmsh, one for each chain.

import gmsh
import sys

gmsh.initialize(sys.argv)

gmsh.option.setNumber("General.Terminal", 1)

# Create an example geometry
gmsh.model.add("t14")

m = 0.5; # mesh characteristic length
h = 2;   # geometry height in the z-direction

gmsh.model.geo.addPoint(0, 0, 0, m, 1)
gmsh.model.geo.addPoint(10, 0, 0, m, 2)
gmsh.model.geo.addPoint(10, 10, 0, m, 3)
gmsh.model.geo.addPoint(0, 10, 0, m, 4)

gmsh.model.geo.addPoint(4, 4, 0, m, 5)
gmsh.model.geo.addPoint(6, 4, 0, m, 6)
gmsh.model.geo.addPoint(6, 6, 0, m, 7)
gmsh.model.geo.addPoint(4, 6, 0, m, 8)

gmsh.model.geo.addPoint(2, 0, 0, m, 9)
gmsh.model.geo.addPoint(8, 0, 0, m, 10)
gmsh.model.geo.addPoint(2, 10, 0, m, 11)
gmsh.model.geo.addPoint(8, 10, 0, m, 12)

gmsh.model.geo.addLine(1, 9, 1)
gmsh.model.geo.addLine(9, 10, 2)
gmsh.model.geo.addLine(10, 2, 3)

gmsh.model.geo.addLine(2, 3, 4)
gmsh.model.geo.addLine(3, 12, 5)
gmsh.model.geo.addLine(12, 11, 6)

gmsh.model.geo.addLine(11, 4, 7)
gmsh.model.geo.addLine(4, 1, 8)
gmsh.model.geo.addLine(5, 6, 9)

gmsh.model.geo.addLine(6, 7, 10)
gmsh.model.geo.addLine(7, 8, 11)
gmsh.model.geo.addLine(8, 5, 12)

gmsh.model.geo.addCurveLoop([6, 7, 8, 1, 2, 3, 4, 5], 13)
gmsh.model.geo.addCurveLoop([11, 12, 9, 10], 14)
gmsh.model.geo.addPlaneSurface([13, 14], 15)

ext_tags = gmsh.model.geo.extrude([(2,15)], 0, 0, h)

# Create physical groups, which are used to define the domain of the
# (co)homology computation and the subdomain of the relative (co)homology
# computation.

# Whole domain
domain_tag = 1;
domain_physical_tag = 1001;
gmsh.model.addPhysicalGroup(dim=3, tags=[domain_tag], tag=domain_physical_tag)
gmsh.model.setPhysicalName(dim=3, tag=domain_physical_tag, name="Whole domain")

# Four "terminals" of the model
terminal_tags = [36, 44, 52, 60];
terminals_physical_tag = 2001
gmsh.model.addPhysicalGroup(dim=2, tags=terminal_tags, tag=terminals_physical_tag)
gmsh.model.setPhysicalName(dim=2, tag=terminals_physical_tag, name="Terminals")

# Find domain boundary tags
boundary_dimtags = gmsh.model.getBoundary(dimTags=[(3, domain_tag)], oriented=False)
boundary_tags = []
complement_tags = []
for tag in boundary_dimtags:
    complement_tags.append(tag[1])
    boundary_tags.append(tag[1])
for tag in terminal_tags:
    complement_tags.remove(tag)

# Whole domain surface
boundary_physical_tag = 2002
gmsh.model.addPhysicalGroup(dim=2, tags=boundary_tags, tag=boundary_physical_tag)
gmsh.model.setPhysicalName(dim=2, tag=boundary_physical_tag, name="Boundary")

# Complement of the domain surface respect to the four terminals
complement_physical_tag = 2003
gmsh.model.addPhysicalGroup(dim=2, tags=complement_tags, tag=complement_physical_tag)
gmsh.model.setPhysicalName(dim=2, tag=complement_physical_tag, name="Complement")

gmsh.model.geo.synchronize()

# Find bases for relative homology spaces of the domain modulo the four
# terminals.
gmsh.model.mesh.computeHomology(domainTags=[domain_physical_tag], subdomainTags=[terminals_physical_tag], dims=[0,1,2,3])

# Find homology space bases isomorphic to the previous bases: homology spaces
# modulo the non-terminal domain surface, a.k.a the thin cuts.
gmsh.model.mesh.computeHomology(domainTags=[domain_physical_tag], subdomainTags=[complement_physical_tag], dims=[0,1,2,3])

# Find cohomology space bases isomorphic to the previous bases: cohomology
# spaces of the domain modulo the four terminals, a.k.a the thick cuts.
gmsh.model.mesh.computeCohomology(domainTags=[domain_physical_tag], subdomainTags=[terminals_physical_tag], dims=[0,1,2,3])

# more examples
#gmsh.model.mesh.computeHomology()
#gmsh.model.mesh.computeHomology(domainTags=[domain_physical_tag])
#gmsh.model.mesh.computeHomology(domainTags=[domain_physical_tag], subdomainTags=[boundary_physical_tag], dims=[0,1,2,3])

# Generate the mesh and perform the requested homology computations
gmsh.model.mesh.generate(3)

# Find physical tags of a certain homology or cohomology space chains
physicals = gmsh.model.getPhysicalGroups()
for pi in physicals:
    name = gmsh.model.getPhysicalName(pi[0], pi[1])
    if(name.find("H^1") == 0): # find tags of all cohomology chains of dimension 1
       print("H^1 tag: " + str(pi[1]) + ", name: " + name)

gmsh.write("t14.msh")
gmsh.finalize()



import gmsh

gmsh.initialize()
gmsh.option.setNumber("General.Terminal", 1)

gmsh.model.add("periodic")
R = 2
gmsh.model.occ.addBox(0, 0, 0, R, R, R)
gmsh.model.occ.synchronize()

ent = gmsh.model.getEntities(0);
gmsh.model.mesh.setSize(ent, 1);
gmsh.model.mesh.setSize([(0,1)], 0.01);
gmsh.model.mesh.setPeriodic(2, [2], [1], [1,0,0,R, 0,1,0,0, 0,0,1,0, 0,0,0,1])

gmsh.model.mesh.generate(2)
#gmsh.model.mesh.setOrder(2)

masterTag, nodes, tfo = gmsh.model.mesh.getPeriodicNodes(2, 2)
# print masterTag, nodes, tfo

gmsh.write("periodic.msh")



"""
ONELAB - Copyright (C) 2011-2019 Universite de Liege - Universite catholique
de Louvain

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, and/or sell copies of the
Software, and to permit persons to whom the Software is furnished to do so,
provided that the above copyright notice(s) and this permission notice appear in
all copies of the Software and that both the above copyright notice(s) and this
permission notice appear in supporting documentation.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR HOLDERS INCLUDED IN THIS NOTICE BE LIABLE FOR ANY
CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

"""

#!/usr/bin/env python

import socket, struct, os, sys, subprocess, time
_VERSION = '1.3'

def path(dirname, inp):
  # dirname is a directory, can be empty
  # inp is an optional file or subdirectory name
  # returns the path to 'inp' in the same directory as 'ref'
  if dirname:
    dirname = os.path.dirname(dirname + os.sep)
  if not inp:
    return dirname
  if inp[0] == '/' or inp[0] == '\\' or (len(inp) > 2 and inp[1] == '\:'):
    return inp # do nothing, inp is an absolute path
  if inp[0] == '.' :
    inp = inp[2:] # cut off heading './' or '.\'
  if dirname:
    return dirname + os.sep + inp # append inp to the path of the reference file
  else:
    return inp

class _parameter() :
  _membersbase = [
    ('name', 'string'), ('label', 'string', ''), ('help', 'string', ''),
    ('changedValue', 'int', 31), ('visible', 'int', 1), ('readOnly', 'int', 0),
    ('attributes', ('dict', 'string', 'string'), {}),
    ('clients', ('dict', 'string', 'int'), {})
  ]
  _members = {
    'string' : _membersbase + [
      ('values', ('list', 'string'), ['']), ('kind', 'string', 'generic'),
      ('choices', ('list', 'string'), [])
    ],
    'number' : _membersbase + [
      ('values', ('list', 'float'), [0.]),
      ('min', 'float', -sys.float_info.max), ('max', 'float', sys.float_info.max),
      ('step', 'float', 0.), ('index', 'int', -1), ('choices', ('list', 'float'), []),
      ('labels', ('dict', 'float', 'string'), {})
    ]
  }

  def __init__(self, type, **values) :
    self.type = type
    for i in _parameter._members[self.type] :
      setattr(self, i[0], values[i[0]] if i[0] in values else i[2])

  def tochar(self) :
    def tocharitem(l, t, v) :
      if t == 'string' : l.append(v)
      elif t =='int': l.append(str(v))
      elif t == 'float' : l.append('%.16g' % v)
      elif t[0] == 'list' :
        l.append(str(len(v)))
        for i in v : tocharitem(l, t[1], i)
      elif t[0] == 'dict' :
        l.append(str(len(v)))
        for i, j in v.items() :
          tocharitem(l, t[1], i)
          tocharitem(l, t[2], j)
    msg = [_VERSION, self.type]
    for i in _parameter._members[self.type] :
      tocharitem(msg, i[1], getattr(self, i[0]))
    return '\0'.join(msg)

  def fromchar(self, msg) :
    def fromcharitem(l, t) :
      if t == 'string' : return l.pop()
      elif t =='int': return int(l.pop())
      elif t == 'float' : return float(l.pop())
      elif t[0] == 'list' : return [fromcharitem(l, t[1]) for i in range(int(l.pop()))]
      elif t[0] == 'dict' : return dict([(fromcharitem(l, t[1]),fromcharitem(l, t[2]))
                                       for i in range(int(l.pop()))])
    l = msg.split('\0')
    l.reverse()
    if l.pop() != _VERSION :
      print('onelab version mismatch')
    if l.pop() != self.type :
      print('onelab parameter type mismatch')
    for p in  _parameter._members[self.type]:
      setattr(self, p[0], fromcharitem(l, p[1]))
    return self

  def modify(self, **param) :
    ## updates the parameter with the content of param, attributes are merged
    for i in _parameter._members[self.type] :
      if i[0] in param :
        if i[0] == 'attributes' :
          self.attributes.update(param['attributes'])
        else :
          setattr(self, i[0], param[i[0]])


class client :
  _GMSH_START = 1
  _GMSH_STOP = 2
  _GMSH_INFO = 10
  _GMSH_WARNING = 11
  _GMSH_ERROR = 12
  _GMSH_PROGRESS = 13
  _GMSH_MERGE_FILE = 20
  _GMSH_PARSE_STRING = 21
  _GMSH_PARAMETER = 23
  _GMSH_PARAMETER_QUERY = 24
  _GMSH_CONNECT = 27
  _GMSH_OLPARSE = 28
  _GMSH_PARAMETER_NOT_FOUND = 29
  _GMSH_PARAMETER_CLEAR = 31
  _GMSH_PARAMETER_UPDATE = 32
  _GMSH_OPEN_PROJECT = 33
  _GMSH_CLIENT_CHANGED = 34

  def _createSocket(self) :
    addr = self.addr
    if '/' in addr or '\\' in addr or ':' not in addr :
      self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
      self.socket.connect(addr)
    else :
      self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
      s = addr.split(':')
      self.socket.connect((s[0], int(s[1])))

  def _receive(self) :
    def buffered_receive(l) :
      msg = b''
      while len(msg) < l:
        chunk = self.socket.recv(l - len(msg))
        if not chunk :
          RuntimeError('onelab socket closed')
        msg += chunk
      return msg
    msg = buffered_receive(struct.calcsize('ii'))
    t, l = struct.unpack('ii', msg)
    msg = buffered_receive(l).decode('utf-8')
    if t == self._GMSH_INFO :
      print('onelab info : %s' % msg)
    return t, msg

  def _send(self, t, msg) :
    if not self.socket :
      return
    m = msg.encode('utf-8')
    try:
      if self.socket.send(struct.pack('ii%is' %len(m), t, len(m), m)) == 0 :
        RuntimeError('onelab socket closed')
    except socket.error:
      self.socket.close()
      self._createSocket()
      self.socket.send(struct.pack('ii%is' %len(m), t, len(m), m))

  def _defineParameter(self, p) :
    if not self.socket :
      return p.values
    self._send(self._GMSH_PARAMETER_QUERY, p.tochar())
    (t, msg) = self._receive()
    if t == self._GMSH_PARAMETER :
      self._send(self._GMSH_PARAMETER_UPDATE, p.tochar())
      return p.fromchar(msg).values
    elif t == self._GMSH_PARAMETER_NOT_FOUND :
      self._send(self._GMSH_PARAMETER, p.tochar())
      return p.values

  def _getParameter(self, param, warn_if_not_found=True) :
    if not self.socket :
      return
    self._send(self._GMSH_PARAMETER_QUERY, param.tochar())
    (t, msg) = self._receive()
    if t == self._GMSH_PARAMETER :
      param.fromchar(msg)
    elif t == self._GMSH_PARAMETER_NOT_FOUND and warn_if_not_found :
      print ('Unknown parameter %s' %(param.name))

  def defineNumber(self, name, **param):
    if 'value' in param :
      param["values"] = [param["value"]]
      del param["value"]
    if 'labels' in param :
      param["choices"] = param["labels"].keys()
    p = _parameter('number', name=name, **param)
    values = self._defineParameter(p)
    return values[0] if len(values) else 0

  def defineString(self, name, **param):
    if 'value' in param :
      param["values"] = [param["value"]]
      del param["value"]
    p = _parameter('string', name=name, **param)
    values = self._defineParameter(p)
    return values[0] if len(values) else ''

  def setNumber(self, name, **param):
    if not self.socket :
      return
    if 'value' in param :
      param["values"] = [param["value"]]
      del param["value"]
    p = _parameter('number', name=name)
    self._send(self._GMSH_PARAMETER_QUERY, p.tochar())
    (t, msg) = self._receive()
    if t == self._GMSH_PARAMETER :
      p.fromchar(msg).modify(**param)
    elif t == self._GMSH_PARAMETER_NOT_FOUND :
      p.modify(**param)
    self._send(self._GMSH_PARAMETER, p.tochar())

  def setString(self, name, **param):
    if not self.socket :
      return
    if 'value' in param :
      param["values"] = [param["value"]]
      del param["value"]
    p = _parameter('string', name=name)
    self._send(self._GMSH_PARAMETER_QUERY, p.tochar())
    (t, msg) = self._receive()
    if t == self._GMSH_PARAMETER : #modify an existing parameter
      p.fromchar(msg).modify(**param)
    elif t == self._GMSH_PARAMETER_NOT_FOUND : #create a new parameter
      p.modify(**param)
    self._send(self._GMSH_PARAMETER, p.tochar())

  def clear(self, name) :
    if not self.socket :
      return
    self._send(self._GMSH_PARAMETER_CLEAR, str(name))

  def _setNumberChoices(self, name, val):
    if not self.socket :
      return
    p = _parameter('number', name=name)
    self._send(self._GMSH_PARAMETER_QUERY, p.tochar())
    (t, msg) = self._receive()
    if t == self._GMSH_PARAMETER :
      if len(val) : # add new choices
        p.fromchar(msg).values = val
        p.choices.extend(val)
      else : # reset choices list
        p.fromchar(msg).choices = ()
    elif t == self._GMSH_PARAMETER_NOT_FOUND :
      print ('Unknown parameter %s' %(param.name))
    self._send(self._GMSH_PARAMETER, p.tochar())

  def resetNumberChoices(self, name):
    self._setNumberChoices(name,[])

  def addNumberChoice(self, name, value):
    self._setNumberChoices(name,[value])

  def getNumber(self, name, warn_if_not_found=True):
    param = _parameter('number', name=name)
    self._getParameter(param, warn_if_not_found)
    return param.values[0] if len(param.values) else 0

  def getNumbers(self, name, warn_if_not_found=True):
    param = _parameter('number', name=name)
    self._getParameter(param, warn_if_not_found)
    return param.values

  def getString(self, name, warn_if_not_found=True):
    param = _parameter('string', name=name)
    self._getParameter(param, warn_if_not_found)
    return param.values[0] if len(param.values) else ''

  def getStrings(self, name, warn_if_not_found=True):
    param = _parameter('string', name=name)
    self._getParameter(param, warn_if_not_found)
    return param.values

  def getNumberChoices(self, name, warn_if_not_found=True):
    param = _parameter('number', name=name)
    self._getParameter(param, warn_if_not_found)
    return param.choices

  def getStringChoices(self, name, warn_if_not_found=True):
    param = _parameter('string', name=name)
    self._getParameter(param, warn_if_not_found)
    return param.choices

  def getNumberRange(self, name, warn_if_not_found=True):
    param = _parameter('number', name=name)
    self._getParameter(param, warn_if_not_found)
    return [param.values[0] if len(param.values) else 0, param.min, param.max]

  def show(self, name) :
    if not self.socket or not name:
      return
    param = _parameter('number', name=name)
    self._send(self._GMSH_PARAMETER_QUERY, param.tochar())
    (t, msg) = self._receive()
    if t == self._GMSH_PARAMETER :
      print (msg.replace('\0','|'))
    elif t == self._GMSH_PARAMETER_NOT_FOUND :
      param = _parameter('string', name=name)
      self._send(self._GMSH_PARAMETER_QUERY, param.tochar())
      (t, msg) = self._receive()
      if t == self._GMSH_PARAMETER :
        print (msg.replace('\0','|'))
      elif t == self._GMSH_PARAMETER_NOT_FOUND :
        print('Unknown parameter %s' %(name))

  def sendCommand(self, command) :
    if not self.socket :
      return
    self._send(self._GMSH_PARSE_STRING, command)

  def mergeFile(self, filename) :
    if not self.socket or not filename :
      return
    self._send(self._GMSH_MERGE_FILE, filename)

  def openProject(self, filename) :
    if not self.socket or not filename :
      return
    self._send(self._GMSH_OPEN_PROJECT, filename)

  def sendInfo(self, msg) :
    if not self.socket :
      print (msg)
      return
    self._send(self._GMSH_INFO, str(msg))

  def sendWarning(self, msg) :
    if not self.socket :
      print (msg)
      return
    self._send(self._GMSH_WARNING, str(msg))

  def sendError(self, msg) :
    if not self.socket :
      print (msg)
      return
    self._send(self._GMSH_ERROR, str(msg))

  def preProcess(self, name, filename) :
    if not self.socket :
      return
    msg = [name, filename]
    self._send(self._GMSH_OLPARSE, '\0'.join(msg))
    (t, msg) = self._receive()
    if t == self._GMSH_OLPARSE :
      if msg == "true" :
          return True
    return False

  def isChanged(self, name) :
    if not self.socket :
      return
    msg = ["get", name]
    self._send(self._GMSH_CLIENT_CHANGED, '\0'.join(msg))
    (t, msg) = self._receive()
    if t == self._GMSH_CLIENT_CHANGED :
      if msg == "true" :
          return True
    return False

  def setChanged(self, name, changed) :
    if not self.socket :
      return
    msg = ["set", name, 'true' if changed else 'false']
    self._send(self._GMSH_CLIENT_CHANGED, '\0'.join(msg))

  def waitOnSubClients(self):
    if not self.socket :
      return
    while self._numSubClients > 0:
      (t, msg) = self._receive()
      if t == self._GMSH_STOP :
        self._numSubClients -= 1

  def runNonBlockingSubClient(self, name, command, arguments=''):
    if self.action == 'check':
      cmd = command
    else:
      cmd = command + ' ' + arguments
    if not self.socket :
      return os.system(cmd);
    msg = [name, cmd]
    self._send(self._GMSH_CONNECT, '\0'.join(msg))
    self._numSubClients +=1

  def runSubClient(self, name, command, arguments=''):
    self.runNonBlockingSubClient(name, command, arguments)
    self.waitOnSubClients() # makes the subclient blocking
    if self.action == 'compute':
      self.setChanged(name, False)

  def run(self, name, command, arguments=''):
    self.runSubClient(name, command, arguments)

  def __init__(self, ref=''):
    self.socket = None
    self.name = ""
    self.addr = ""
    self.wdir = os.path.dirname(ref)
    self._numSubClients = 0
    for i, v in enumerate(sys.argv) :
      if v == '-onelab':
        self.name = sys.argv[i + 1]
        self.addr = sys.argv[i + 2]
        self._createSocket()
        self._send(self._GMSH_START, str(os.getpid()))
    self.action = self.getString(self.name + '/Action', False)
    self.setNumber('IsPyMetamodel',values=[1],visible=0)
    self.loop = self.getNumber('0Metamodel/Loop', warn_if_not_found=False)
    self.batch = self.getNumber('0Metamodel/Batch', warn_if_not_found=False)
    self.sendInfo("Performing OneLab '" + self.action + "'")
    if self.action == "initialize":
      self.finalize()
      exit(0)

  def finalize(self):
    # code aster python interpreter does not call the destructor at exit, it is
    # necessary to call finalize() epxlicitely
    if self.socket :
      self.waitOnSubClients()
      self._send(self._GMSH_STOP, 'Goodbye!')
      self.socket.close()
      self.socket = None

  def __del__(self):
    self.finalize()

  def call(self, name, cmdline, remote='', rundir='', logfile='', poll=0):
    cwd = None
    if not remote :
      argv = cmdline.rsplit(' ')
      if rundir :
        cwd = rundir
    else :
      argv=['ssh', remote , "cd %s ; %s" %(rundir,cmdline) ]

    if poll and not logfile :
        logfile = os.devnull
    if logfile:
      call = subprocess.Popen(argv, bufsize=1, cwd=cwd,
                              stdout=open(logfile,"w"),
                              stderr=subprocess.STDOUT)
      while call.poll() == None :
        self.action = self.getString(self.name + '/Action', False)
        if self.action == 'stop':
          call.terminate()
          self._send(self._GMSH_WARNING, 'client killed')
          sys.exit(1)
        time.sleep(1) # check every second for the stop signal from Gmsh
    else:
      call = subprocess.Popen(argv, bufsize=1, cwd=cwd,
                              stdout=subprocess.PIPE,
                              stderr=subprocess.STDOUT)
      count = 0
      for line in iter(call.stdout.readline, b''):
        print(line.rstrip())
        count += 1
        if count == 50 : # check every 50 lines for the stop signal from Gmsh
          count = 0
          self.action = self.getString(self.name + '/Action', False)
          if self.action == 'stop':
            call.terminate()
            self._send(self._GMSH_WARNING, 'client killed')
            sys.exit(1)

    result = call.wait()
    if result == 0 :
      self._send(self._GMSH_INFO, 'done \"' + ' '.join(argv) + '\"')
      if self.action == 'compute':
        self.setChanged(name, False)
    else :
      self._send(self._GMSH_ERROR, 'error \"' + ' '.join(argv) + '\"')
      sys.exit(1)

  def copy(self, here, there):
    os.system('cp '+ here + ' ' + there)

  def upload(self, here, there, remote='') :
    if not here or not there :
      return
    if remote :
      argv=['rsync','-e','ssh','-auv', here, remote + ':' + there]
    else :
      argv=['cp','-f', here, there]

    call = subprocess.Popen(argv, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
    result = call.wait()
    if result == 0 :
      self._send(self._GMSH_INFO, 'upload: ' + ' '.join(argv))
    else :
      print(call.stderr.read())

  def download(self, there, here, remote='') :
    if not here or not there :
      return
    if remote :
      argv=['rsync','-e','ssh','-auv', remote + ':' + there, here]
    else :
      argv=['cp','-f', there, here]

    call = subprocess.Popen(argv, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
    result = call.wait()
    if result == 0 :
      self._send(self._GMSH_INFO, 'download: ' + ' '.join(argv))
    else :
      print(call.stderr.read())

  def getPath(self, inp='') :
    return path(self.wdir, inp)

  def fileExists(self, p) :
    return os.path.exists(p)

  def checkFile(self, p) :
    if not self.fileExists(p):
      self.sendError('path error: %s' %(p))
      exit(0)
    return True

  def checkPath(self, inp='') :
    p = path(self.wdir,inp)
    self.checkFile(p)
    return p

  def solutionFiles(self, client, list) :
    self.defineNumber('0Metamodel/9Use restored solution', values=[0], choices=[0,1])
    self.defineString('0Metamodel/9Tag', values=[''])
    if list :
      if self.getNumber('0Metamodel/9Use restored solution') :
        solFiles = self.getStringChoices('0Metamodel/9Solution files')
        for i in solFiles:
          self.checkFile(i)
      else :
        solFiles = list
        self.setString('0Metamodel/9Solution files', values=[solFiles[0]],
                       choices=solFiles, readOnly=1)
        for i in solFiles:
          if not self.fileExists(i) :
            self.setChanged(client, 'true')
    return solFiles

  def needsCompute(self, client) :
    # Avoid recomputation when a database with restored solutions has just been loaded
    # Otherwise, stop using restored solutions when the client elmer has changed
    if self.useRestoredSolution() == 2: # we have just loaded a database
      self.setRestoredSolution(1)
      self.setChanged(client, 'false') # do not compute
    else :
      if self.isChanged(client):
        self.setRestoredSolution(0)
        return True
    return False

  def willCompute(self, client) : # Inform user about the value of needsCompute
    if self.useRestoredSolution() == 2: # special value
      return False
    else :
      return True if self.isChanged(client) else False

  def useRestoredSolution(self) :
    return self.getNumber('0Metamodel/9Use restored solution')

  def setRestoredSolution(self, val) :
    self.setNumber('0Metamodel/9Use restored solution', values=[val])

  def outputFiles(self, list) :
    if list :
      self.setString('0Metamodel/9Output files', values=[list[0]],
                     choices=list, visible=1)

# tool to extract the (i, j)th element in an array file
# from rlcompleter import readline
def extract(filename,i,j):
    input = open(filename,'r')
    all_lines = input.readlines()
    input.close()
    if i == -1:
        i = len(all_lines) # last line
    items = all_lines[i-1].split()
    return float(items[j-1])



import gmsh
import sys

if len(sys.argv) < 2:
    print "Usage: " + sys.argv[0] + " file.msh [options]"
    exit(0)

gmsh.initialize()
gmsh.option.setNumber("General.Terminal", 1)
gmsh.open(sys.argv[1])

# get all elementary entities in the model
entities = gmsh.model.getEntities()

for e in entities:
    # get the mesh nodes for each elementary entity
    nodeTags, nodeCoords, nodeParams = gmsh.model.mesh.getNodes(e[0], e[1])
    # get the mesh elements for each elementary entity
    elemTypes, elemTags, elemNodeTags = gmsh.model.mesh.getElements(e[0], e[1])
    # report some statistics
    numElem = sum(len(i) for i in elemTags)
    print str(len(nodeTags)) + " mesh nodes and " + str(numElem),\
          "mesh elements on entity " + str(e) + " of type " + gmsh.model.getType(e[0], e[1])
    partitions = gmsh.model.getPartitions(e[0], e[1])
    if len(partitions):
           print " - Partition tag(s): " + str(partitions) + " - parent entity" +\
               str(gmsh.model.getParent(e[0], e[1]))
    for t in elemTypes:
        name, dim, order, numv, parv = gmsh.model.mesh.getElementProperties(t)
        print " - Element type: " + name + ", order " + str(order)
        print "   with " + str(numv) + " nodes in param coord: ", parv

# all mesh node coordinates
nodeTags, nodeCoords, _ = gmsh.model.mesh.getNodes()
x = dict(zip(nodeTags, nodeCoords[0::3]))
y = dict(zip(nodeTags, nodeCoords[1::3]))
z = dict(zip(nodeTags, nodeCoords[2::3]))

gmsh.finalize()



import gmsh
import sys

gmsh.initialize(sys.argv)
gmsh.option.setNumber("General.Terminal", 1)

# Copied from discrete.py...
gmsh.model.add("test");
gmsh.model.addDiscreteEntity(2, 1)
gmsh.model.mesh.setNodes(2, 1, [1, 2, 3, 4],
                         [0., 0., 0.,
                          1., 0., 0.,
                          1., 1., 0.,
                          0., 1., 0.])
gmsh.model.mesh.setElements(2, 1, [2], [[1, 2]],
                            [[1, 2, 3,
                              1, 3, 4]])
# ... end of copy

# create a view with some data
t = gmsh.view.add("some data")
gmsh.view.addModelData(t, 0, "test", "NodeData",
                       [1, 2, 3, 4],
                       [[1.],[10.],[20.],[1.]])

# test getting data back
dataType, tags, data, time, numComp = gmsh.view.getModelData(t, 0)
print dataType, tags

# compute the iso-curve at value 11
gmsh.plugin.setNumber("Isosurface", "Value", 11.)
gmsh.plugin.run("Isosurface")

# delete the source view
gmsh.view.remove(t)

# check how many views the plugin created (a priori, a single list-based one)
tags = gmsh.view.getTags()
if len(tags) == 1:
    gmsh.view.write(tags[0], "iso.msh")
    # test getting data back
    dataTypes, numElements, data = gmsh.view.getListData(tags[0])
    print dataTypes, numElements

gmsh.finalize()



import gmsh
import numpy as np
import scipy.sparse
import scipy.sparse.linalg
import sys

# This scripts solves the 2D Poisson equation '-\Delta u = f' with homogeneous
# boundary conditions using the finite element method.

# Simply run the script with
# $ python demos/api/poisson.py
# with usual gmsh line arguments, e.g., -clscale 0.5 -order 2

INTEGRATION = 'Gauss2'
DEBUG = 0
RECOMBINE = 0

def debug(*args):
    if not DEBUG: return
    if sys.version_info.major == 3:
        exec("print( *args )")
    else:
        for item in args: exec("print item,")
        exec("print")

def error(argv):
    debug(argv)
    exit(1)

def create_geometry():
    model.add("poisson")
    surf = []
    surf.append((2, factory.addRectangle(0, 0, 0, 1, 1)))
    surf.append((2, factory.addDisk(0.7, 0.5, 0, 0.1, 0.1)))
    surf.append((2, factory.addRectangle(0.2, 0.4, 0, 0.2, 0.2)))
    surf, _ = factory.fragment(surf, [])
    factory.synchronize()

    bnd = model.getBoundary(surf, combined=True, oriented=False, recursive=False)
    bnd = np.array(bnd)

    model.addPhysicalGroup(2, [surf[0][1]], 2)
    model.setPhysicalName(2, 2, 'SourceCircle')
    model.addPhysicalGroup(2, [surf[1][1]], 3)
    model.setPhysicalName(2, 3, 'SourceSquare')
    model.addPhysicalGroup(2, [surf[2][1]], 4)
    model.setPhysicalName(2, 4, 'Domain')
    model.addPhysicalGroup(1, bnd[:,1], 11)
    model.setPhysicalName(1, 11, 'Boundary')

    model.mesh.setSize(model.getEntities(0), 0.1);
    return

def fem_solve():
    mshNodes = np.array(model.mesh.getNodes()[0])
    numMeshNodes = len(mshNodes)
    maxNodeTag = np.amax(mshNodes)
    debug('numMeshNodes =', numMeshNodes, ' maxNodeTag =', maxNodeTag)

    # typNodes[tag] = {0:does not exist, 1:internal node, 2:boundary node}
    # Existing node tags are defined here. Boundary node tag are identified later.
    typNodes = np.zeros(maxNodeTag+1, dtype=np.int32)
    for tagNode in mshNodes:
        typNodes[tagNode] = 1

    # initializations of global assembly arrays iteratively filled-in during
    # assembly
    matrowflat = np.array([], dtype=np.int32)
    matcolflat = np.array([], dtype=np.int32)
    matflat    = np.array([], dtype=np.int32)
    rhsrowflat = np.array([], dtype=np.int32)
    rhsflat    = np.array([], dtype=np.int32)

    # The read-in mesh is iterated over, looping successively (nested loops) over:
    # Physical groups/Geometrical entities/Element types/Elements
    vGroups = model.getPhysicalGroups()
    for iGroup in vGroups:
        dimGroup = iGroup[0] # 1D, 2D or 3D
        tagGroup = iGroup[1]
        namGroup = model.getPhysicalName(dimGroup, tagGroup)
        vEntities = model.getEntitiesForPhysicalGroup(dimGroup, tagGroup)
        for tagEntity in vEntities:
            # FIXME dimEntity should be optional when tagEntity given
            dimEntity = dimGroup
            vElementTypes = model.mesh.getElementTypes(dimEntity,tagEntity)
            for elementType in vElementTypes:
                vTags, vNodes = model.mesh.getElementsByType(elementType, tagEntity)
                numElements = len(vTags)
                numGroupNodes = len(vNodes)
                enode = np.array(vNodes).reshape((numElements,-1))
                numElementNodes = enode.shape[1]
                debug('\nIn group', namGroup, ' with tag ', tagGroup,
                      ', numElements = e =', numElements)
                debug('numGroupNodes =', numGroupNodes,', numElementNodes = n =',
                      numElementNodes)
                debug('%enodes (e,n) =', enode.shape)

                # Assembly of stiffness matrix for all 2 dimensional elements
                # (triangles or quadrangles)
                if dimEntity==2 :
                    uvwo, numcomp, sf = model.mesh.getBasisFunctions(
                        elementType, INTEGRATION, 'Lagrange')
                    # debug('%uvwo =', len(uvwo), '%numcomp =', numcomp,
                    #       '%sf =', len(sf))
                    # only keep the Gauss weights
                    weights = np.array(uvwo).reshape((-1,4))[:,3]
                    numGaussPoints = weights.shape[0]
                    debug('numGaussPoints = g =', numGaussPoints, ', %weights (g) =',
                          weights.shape)
                    sf = np.array(sf).reshape((numGaussPoints, -1))
                    debug('%sf (g,n) =', sf.shape)
                    if sf.shape[1] != numElementNodes:
                        error('Something went wrong')
                    _, numcomp, dsfdu = model.mesh.getBasisFunctions(
                        elementType, INTEGRATION, 'GradLagrange')
                    # debug('%uvwo =', len(uvwo), '%numcomp =', numcomp, '%dsfdu =',
                    #       len(dsfdu))
                    # remove useless dsfdw
                    dsfdu = np.array(dsfdu).reshape(
                        (numGaussPoints,numElementNodes,3))[:,:,:-1]
                    debug('%dsfdu (g,n,u) =', dsfdu.shape)

                    qjac, qdet, qpoint = model.mesh.getJacobians(
                        elementType, INTEGRATION, tagEntity)
                    debug('Gauss integr:', len(qjac), len(qdet), len(qpoint),
                           '= (9, 1, 3) x', numGaussPoints, 'x', numElements)
                    qdet = np.array(qdet).reshape((numElements, numGaussPoints))
                    debug('%qdet (e,g) =', qdet.shape)
                    # remove components of dxdu useless in dimEntity dimensions (here 2D)
                    dxdu = np.array(qjac).reshape(
                        (numElements, numGaussPoints, 3, 3))[:,:,:-1,:-1]
                    # jacobien store by row, so dxdu[i][j] = dxdu_ij = dxi/duj
                    debug('%dxdu (e,g,x,u)=', dxdu.shape)

                    # dudx[j][k] = dudx_jk = duj/dxk
                    dudx = np.linalg.inv(dxdu)
                    debug('%dudx (e,g,u,x) =', dudx.shape)
                    # sum over u = dot product
                    dsfdx  = np.einsum("egxu,gnu->egnx",dudx,dsfdu);
                    debug('%dsfdx (e,g,n,x) =', dsfdx.shape)
                    # Gauss integration
                    localmat = np.einsum("egik,egjk,eg,g->eij", dsfdx, dsfdx, qdet,
                                         weights)
                    debug('%localmat (e,n,n) =', localmat.shape)

                    # The next two lines are rather obscure. See explanations at
                    # the bottom of the file.
                    matcol = np.repeat(enode[:,:,None],numElementNodes,axis=2)
                    matrow = np.repeat(enode[:,None,:],numElementNodes,axis=1)

                    matcolflat = np.append(matcolflat, matcol.flatten())
                    matrowflat = np.append(matrowflat, matrow.flatten())
                    matflat = np.append(matflat, localmat.flatten())

                    if namGroup == 'SourceCircle' or namGroup == 'SourceSquare':
                        if namGroup == 'SourceCircle':
                            load = -1
                        else:
                            load = 1
                        localrhs = load * np.einsum("gn,eg,g->en", sf, qdet, weights)
                        rhsrowflat = np.append(rhsrowflat, enode.flatten())
                        rhsflat = np.append(rhsflat, localrhs.flatten())

                # identify boundary node
                if namGroup == 'Boundary':
                    for tagNode in vNodes:
                        typNodes[tagNode] = 2

    # Associate to all mesh nodes a line number in the system matrix, reserving
    # top lines for internal nodes and bottom lines for fixed nodes (boundary
    # nodes).
    node2unknown = np.zeros(maxNodeTag+1, dtype=np.int32)
    index = 0
    for tagNode,typ in enumerate(typNodes):
        if  typ == 1: # not fixed
            index += 1
            node2unknown[tagNode] = index
    numUnknowns = index
    debug('numUnknowns =', numUnknowns)
    for tagNode,typ in enumerate(typNodes):
        if  typ == 2: # fixed
            index += 1
            node2unknown[tagNode] = index

    if index != numMeshNodes:
        error('Something went wrong')

    unknown2node = np.zeros(numMeshNodes+1, dtype=np.int32)
    for node, unkn in enumerate(node2unknown):
        unknown2node[unkn] = node

    # Generate system matrix A=globalmat and right hand side b=globalrhs

    # docs.scipy.org/doc/scipy/reference/generated/scipy.sparse.coo_matrix.html
    # 'node2unknown-1' are because python numbers rows and columns from 0
    globalmat = scipy.sparse.coo_matrix(
        (matflat, (node2unknown[matcolflat]-1,node2unknown[matrowflat]-1) ),
        shape=(numMeshNodes, numMeshNodes)).tocsr()

    globalrhs = np.zeros(numMeshNodes)
    for index,node in enumerate(rhsrowflat):
        globalrhs[node2unknown[node]-1] += rhsflat[index]

    debug('%globalmat =', globalmat.shape, ' %globalrhs =', globalrhs.shape)

    # Solve linear system Ax=b
    sol = scipy.sparse.linalg.spsolve(globalmat[:numUnknowns,:numUnknowns],
                                      globalrhs[:numUnknowns])
    sol = np.append(sol, np.zeros(numMeshNodes-numUnknowns))
    debug('%sol =', sol.shape)

    # Export solution
    sview = gmsh.view.add("solution")
    gmsh.view.addModelData(sview, 0, "", "NodeData", unknown2node[1:],
                           sol[:,None])
    return


model = gmsh.model
factory = model.occ
gmsh.initialize(sys.argv)
create_geometry()

if RECOMBINE:
    model.mesh.setRecombine(2,2)
    model.mesh.setRecombine(2,3)
    model.mesh.setRecombine(2,4)

model.mesh.generate(2)

if DEBUG:
    gmsh.write('poisson.msh')

fem_solve()

gmsh.option.setNumber("View[0].IntervalsType", 3)
gmsh.option.setNumber("View[0].NbIso", 20)
gmsh.fltk.run()

gmsh.finalize()

## Explanation for the serialization of elementary matrices.
##
## node = numpy.array([[1,2,3]]) # one element with nodes 1,2 and 3
## col = numpy.repeat(enode[:,:,None],3,axis=2)
## row = numpy.repeat(enode[:,None,:],3,axis=1)
## >>> col
## array([[[1, 1, 1],
##         [2, 2, 2],
##         [3, 3, 3]]])
## >>> row
## array([[[1, 2, 3],
##         [1, 2, 3],
##         [1, 2, 3]]])



import gmsh
import math
import time
import threading

# This example shows how to implement a custom user interface running
# computationally expensive calculations in separate threads. The threads can
# update the user interface in real-time.

gmsh.initialize()

# hide the standard Gmsh modules
gmsh.option.setNumber("General.ShowModuleMenu", 0)

# create some ONELAB parameters to control the number of iterations and
# threads, the progress display and the custom ONELAB button (when pressed,
# it will set the "ONELAB/Action" parameter to "should compute")
parameters = """
[
  { "type":"number", "name":"My App/Iterations", "values":[1e6],
    "attributes":{"Highlight":"AliceBlue"} },
  { "type":"number", "name":"My App/Number of threads", "values":[2],
    "min":1, "max":16, "step":1, "attributes":{"Highlight":"AliceBlue"} },
  { "type":"number", "name":"My App/Show progress?", "values":[1],
    "choices":[0, 1] },
  { "type":"string", "name":"ONELAB/Button", "values":["Do it!", "should compute"],
    "visible":false }
]"""
gmsh.onelab.set(parameters)

# flag that will be set to interrupt a calculation
stop_computation = False

# a computationally expensive routine, that will be run in its own thread
def compute(arg):
    iterations = gmsh.onelab.getNumber("My App/Iterations")
    progress= gmsh.onelab.getNumber("My App/Show progress?")
    n = int(iterations[0]) if len(iterations) > 0 else 1
    show = True if (len(progress) > 0 and progress[0] == 1) else False
    p = 0
    k = 0
    last_refresh = -1
    for j in range(n):
        # stop computation if requested by clicking on "Stop it!"
        if stop_computation:
            break
        k = math.sin(k) + math.cos(j/45.)
        # show progress in real time?
        if (show == 1) and (n > 1) and (not j % (n / 100)):
            p = p + 1
            gmsh.onelab.setString(arg, ["{0}%".format(p)])
            # any code in a thread other than the main thread that modifies the
            # user interface should be locked
            gmsh.fltk.lock()
            gmsh.logger.write("{0} progress {1}%".format(arg, p))
            gmsh.fltk.unlock()
            # ask the main thread to process pending events and to update
            # the user interface
            if time.time() - last_refresh > 0.1:
                last_refresh = time.time()
                gmsh.fltk.awake("update")
    gmsh.onelab.setNumber(arg + " result", [k])
    gmsh.onelab.setString("ONELAB/Action", ["done computing"])
    gmsh.fltk.awake("update")
    return

# create the graphical user interface
gmsh.fltk.initialize()

while 1:
    # wait for an event
    gmsh.fltk.wait()

    # check if the user clicked on the custom ONELAB button by examining the
    # value of the "ONELAB/Action" parameter
    action = gmsh.onelab.getString("ONELAB/Action")
    if len(action) < 1:
        continue
    elif action[0] == "should compute":
        gmsh.onelab.setString("ONELAB/Action", [""])
        gmsh.onelab.setString("ONELAB/Button", ["Stop!", "should stop"])
        # force interface update (to show the new button label)
        gmsh.fltk.update()
        # start computationally intensive calculations in their own threads
        n = int(gmsh.onelab.getNumber("My App/Number of threads")[0])
        for i in range(n):
            t = threading.Thread(target=compute,
                                 args=("My App/Thread {0}".format(i + 1),))
            t.start()
    elif action[0] == "should stop":
        stop_computation = True
    elif action[0] == "done computing":
        gmsh.onelab.setString("ONELAB/Action", [""])
        gmsh.onelab.setString("ONELAB/Button", ["Do it!", "should compute"])
        gmsh.fltk.update()
        stop_computation = False
    elif action[0] == "reset":
        # user clicked on "Reset database"
        gmsh.onelab.setString("ONELAB/Action", [""])
        gmsh.onelab.set(parameters)
        gmsh.fltk.update()
    elif action[0] == "check":
        # could perform action here after each change in ONELAB parameters,
        # e.g. rebuild a CAD model, update other parameters, ...
        continue

gmsh.finalize()



# This reimplements gmsh/demos/boolean/boolean.geo in Python.

import gmsh
import sys

model = gmsh.model
factory = model.occ

gmsh.initialize(sys.argv)

gmsh.option.setNumber("General.Terminal", 1)

model.add("boolean")

# from http://en.wikipedia.org/wiki/Constructive_solid_geometry

gmsh.option.setNumber("Mesh.Algorithm", 6);
gmsh.option.setNumber("Mesh.CharacteristicLengthMin", 0.4);
gmsh.option.setNumber("Mesh.CharacteristicLengthMax", 0.4);

R = 1.4; Rs = R*.7; Rt = R*1.25

factory.addBox(-R,-R,-R, 2*R,2*R,2*R, 1)
factory.addSphere(0,0,0,Rt, 2)
factory.intersect([(3, 1)], [(3, 2)], 3)
factory.addCylinder(-2*R,0,0, 4*R,0,0, Rs, 4)
factory.addCylinder(0,-2*R,0, 0,4*R,0, Rs, 5)
factory.addCylinder(0,0,-2*R, 0,0,4*R, Rs, 6)
factory.fuse([(3, 4), (3, 5)], [(3, 6)], 7)
factory.cut([(3, 3)], [(3, 7)], 8)

factory.synchronize();

model.mesh.generate(3)
#model.mesh.refine()
#model.mesh.setOrder(2)
#model.mesh.partition(4)

gmsh.write("boolean.msh")

gmsh.finalize()




import gmsh
import sys
import math

gmsh.initialize(sys.argv)
gmsh.option.setNumber("General.Terminal", 1)

gmsh.model.add("2d surface mesh with purely discrete boundary");

# create a discrete curve with N nodes and N line elements
gmsh.model.addDiscreteEntity(1, 100)
N = 50
dt = 2*math.pi/N
pts = [[math.cos(i * dt), math.sin(i * dt), 0] for i in range(N)]
flat_pts = [item for sublist in pts for item in sublist]
gmsh.model.mesh.setNodes(1, 100, range(1, N+1), flat_pts)
n = [item for sublist in [[i, i+1] for i in range(1, N+1)] for item in sublist]
n[-1] = 1
gmsh.model.mesh.setElements(1, 100, [1], [range(1, N+1)], [n])

# create a plane surface from the discrete curve
gmsh.model.geo.addCurveLoop([100], 101)
gmsh.model.geo.addPlaneSurface([101], 102)
gmsh.model.geo.synchronize()

# mesh the surface
gmsh.model.mesh.generate(2)
gmsh.fltk.run()

gmsh.finalize()



import gmsh
import sys

gmsh.initialize(sys.argv)
gmsh.fltk.initialize()

gmsh.option.setNumber("General.Terminal", 1)

gmsh.model.add("square with cracks")

surf1 = 1
gmsh.model.occ.addRectangle(0, 0, 0, 1, 1, surf1)

pt1 = gmsh.model.occ.addPoint(0.2, 0.2, 0)
pt2 = gmsh.model.occ.addPoint(0.4, 0.4, 0)
line1 = gmsh.model.occ.addLine(pt1, pt2)
pt3 = gmsh.model.occ.addPoint(0.6, 0.1, 0)
pt4 = gmsh.model.occ.addPoint(0.1, 0.3, 0)
line2 = gmsh.model.occ.addLine(pt3, pt4)

o, m = gmsh.model.occ.fragment([(2,surf1)], [(1,line1), (1,line2)])
gmsh.model.occ.synchronize()

# m contains, for each input entity (surf1, line1 and line2), the child entities
# (if any) after the fragmentation, as lists of tuples. To apply the crack
# plugin we group all the intersecting lines in a physical group

new_surf = m[0][0][1]
new_lines = [item[1] for sublist in m[1:] for item in sublist]

gmsh.model.addPhysicalGroup(2, [new_surf], 100)
gmsh.model.addPhysicalGroup(1, new_lines, 101)

gmsh.model.mesh.generate(2)

gmsh.plugin.setNumber("Crack", "PhysicalGroup", 101)
gmsh.plugin.run("Crack")

gmsh.fltk.run()

gmsh.finalize()



import numpy as np
import sys
import gmsh


def triangle_max_edge(x):
    a = np.sum((x[:,0,:]-x[:,1,:])**2,1)**0.5
    b = np.sum((x[:,0,:]-x[:,2,:])**2,1)**0.5
    c = np.sum((x[:,1,:]-x[:,2,:])**2,1)**0.5
    return np.maximum(a,np.maximum(b,c))


class Mesh:

    def __init__(self):
        self.vtags, vxyz, _ = gmsh.model.mesh.getNodes()
        self.vxyz = vxyz.reshape((-1,3))
        vmap = dict({j:i for i,j in enumerate(self.vtags)})
        self.triangles_tags, evtags = gmsh.model.mesh.getElementsByType(2)
        evid = np.array([vmap[j] for j in evtags])
        self.triangles = evid.reshape((self.triangles_tags.shape[-1],-1))


def my_function(xyz):
    a = 6*(np.hypot(xyz[...,0]-.5,xyz[...,1]-.5)-.2)
    f = np.real(np.arctanh(a+0j))
    return f


def compute_interpolation_error(nodes, triangles, f):
    jac,det,pt = gmsh.model.mesh.getJacobians(2,"Gauss2")
    uvwo,numcomp,sf = gmsh.model.mesh.getBasisFunctions(2,"Gauss2","Lagrange")
    weights = uvwo.reshape([-1,4])[:,3]
    sf = sf.reshape((weights.shape[0],-1))
    qx = pt.reshape((triangles.shape[0],-1,3))
    det = np.abs(det.reshape((triangles.shape[0],-1)))
    f_vert = f(nodes)
    f_fem = np.dot(f_vert[triangles],sf)
    err_tri = np.sum((f_fem-f(qx))**2*det*weights,1)
    return f_vert, np.sqrt(err_tri)


def compute_size_field(nodes, triangles, err, N):
    x = nodes[triangles]
    a = 2.
    d = 2.
    fact = (a**((2.+a)/(1.+a)) + a**(1./(1.+a))) * np.sum(err**(2./(1.+a)))
    ri = err**(2./(2.*(1+a))) * a**(1./(d*(1.+a))) * ((1.+a)*N/fact)**(1./d)
    return triangle_max_edge(x)/ri


print("Usage: adapt_mesh [intial lc] [target #elements] [dump files]")

lc = 0.02;
N = 10000;
dumpfiles = False
if len(sys.argv) > 1: lc = float(sys.argv[1])
if len(sys.argv) > 2: N = int(sys.argv[2])
if len(sys.argv) > 3: dumpfiles = int(sys.argv[3])

gmsh.initialize()
gmsh.option.setNumber("General.Terminal", 1)

# create a geometrical gmsh.model
gmsh.model.add("square")
square = gmsh.model.occ.addRectangle(0, 0, 0, 1, 1)
gmsh.model.occ.synchronize()

# create intial uniform mesh
pnts = gmsh.model.getBoundary([(2,square)], True, True, True);
gmsh.model.mesh.setSize(pnts, lc)
gmsh.model.mesh.generate(2)
if dumpfiles: gmsh.write("mesh.msh")
mesh = Mesh()

# compute and visualize the interpolation error
f_nod, err_ele = compute_interpolation_error(mesh.vxyz, mesh.triangles, my_function)
f_view = gmsh.view.add("nodal function")
gmsh.view.addModelData(f_view, 0, "square", "NodeData",
                       mesh.vtags, f_nod[:,None])
if dumpfiles: gmsh.view.write(f_view, "f.pos")
err_view = gmsh.view.add("element-wise error")
gmsh.view.addModelData(err_view, 0, "square", "ElementData",
                       mesh.triangles_tags, err_ele[:,None])
if dumpfiles: gmsh.view.write(err_view, "err.pos")

# compute and visualize the remeshing size field
sf_ele = compute_size_field(mesh.vxyz,mesh.triangles, err_ele, N)
sf_view = gmsh.view.add("mesh size field")
gmsh.view.addModelData(sf_view, 0, "square", "ElementData",
                       mesh.triangles_tags, sf_ele[:,None])
if dumpfiles: gmsh.view.write(sf_view, "sf.pos")

# create a new gmsh.model (to remesh the original gmsh.model in-place, the size field
# should be created as a list-based view)
gmsh.model.add("square2")
gmsh.model.occ.addRectangle(0, 0, 0, 1, 1)
gmsh.model.occ.synchronize()

# mesh the new gmsh.model using the size field
bg_field = gmsh.model.mesh.field.add("PostView");
gmsh.model.mesh.field.setNumber(bg_field, "ViewTag", sf_view);
gmsh.model.mesh.field.setAsBackgroundMesh(bg_field);
gmsh.model.mesh.generate(2)
if dumpfiles: gmsh.write("mesh2.msh")
mesh2 = Mesh()

# compute and visualize the interpolation error on the adapted mesh
f2_nod, err2_ele = compute_interpolation_error(mesh2.vxyz,mesh2.triangles, my_function)
f2_view = gmsh.view.add("nodal function on adapted mesh")
gmsh.view.addModelData(f2_view, 0, "square2", "NodeData",
                       mesh2.vtags, f2_nod[:,None])
if dumpfiles: gmsh.view.write(f2_view, "f2.pos")
err2_view = gmsh.view.add("element-wise error on adapated mesh")
gmsh.view.addModelData(err2_view, 0, "square2", "ElementData",
                       mesh2.triangles_tags, err2_ele[:,None])
if dumpfiles: gmsh.view.write(err2_view, "err2.pos")

# show everything in the gui
gmsh.fltk.run()

gmsh.finalize()



import gmsh
import sys

gmsh.initialize(sys.argv)
gmsh.option.setNumber("General.Terminal", 1)
print(gmsh.option.getNumber("Mesh.Algorithm"))

gmsh.open("square.msh")

model = gmsh.model

model.add("square")
factory = model.geo
factory.addPoint(0,0,0,0.1,1)
factory.addPoint(1,0,0,0.1,2)
factory.addPoint(1,1,0,0.1,3)
factory.addPoint(0,1,0,0.1,4)
factory.addLine(1,2,1)
factory.addLine(2,3,2)
factory.addLine(3,4,3)
line4 = factory.addLine(4,1)
print("line4 received tag ", line4)
factory.addCurveLoop([1,2,3,line4],1)
factory.addPlaneSurface([1],6)
factory.synchronize()

ptag = model.addPhysicalGroup(1,[1,2,3,4])
ent = model.getEntitiesForPhysicalGroup(1,ptag)
print("new physical group ",ptag,":",ent, type(ent))

model.addPhysicalGroup(2,[6])

print(gmsh.option.getString("General.BuildOptions"))
print(gmsh.option.getNumber("Mesh.Algorithm"))
gmsh.option.setNumber("Mesh.Algorithm", 3.0)
print(gmsh.option.getNumber("Mesh.Algorithm"))
model.mesh.generate(2)

gmsh.write("square.msh")

print("Entities")
entities = model.getEntities()
for e in entities :
    print("entity ",e)
    types,tags,nodes = model.mesh.getElements(e[0],e[1])
    for i in range(len(types)):
        print("type ", types[i])
        print("tags : ", list(tags[i]))
        print("nodes : ", list(nodes[i]))
    if e[0] == [2] and e[1] == 6 :
        model.mesh.setElements(e[0],e[1],types,[tags[0][:10]],[nodes[0][:30]])

gmsh.write("mesh_truncated.msh")
print("Nodes")
tags, coord, _ = model.mesh.getNodes(2,6)
print(tags)
print(coord)
gmsh.finalize()



import gmsh
import sys

if len(sys.argv) < 2:
    print "Usage: " + sys.argv[0] + " file.geo [options]"
    exit(0)

gmsh.initialize()
gmsh.option.setNumber("General.Terminal", 1)
gmsh.open(sys.argv[1])
gmsh.model.mesh.generate(3)
gmsh.write("test.msh")
gmsh.finalize()




# This file reimplements gmsh/tutorial/t10.geo in Python.

import gmsh
import math

model = gmsh.model
factory = model.geo

gmsh.initialize()
gmsh.option.setNumber("General.Terminal", 1)

model.add("t10")

lc = .15
factory.addPoint(0.0,0.0,0, lc, 1)
factory.addPoint(1,0.0,0, lc, 2)
factory.addPoint(1,1,0, lc, 3)
factory.addPoint(0,1,0, lc, 4)
factory.addPoint(0.2,.5,0, lc, 5)

factory.addLine(1,2, 1);
factory.addLine(2,3, 2);
factory.addLine(3,4, 3);
factory.addLine(4,1, 4);

factory.addCurveLoop([1,2,3,4], 5)
factory.addPlaneSurface([5], 6)

model.mesh.field.add("Distance", 1)
model.mesh.field.setNumbers(1, "NodesList", [5])
model.mesh.field.setNumber(1, "NNodesByEdge", 100)
model.mesh.field.setNumbers(1, "EdgesList", [2])

model.mesh.field.add("Threshold", 2);
model.mesh.field.setNumber(2, "IField", 1);
model.mesh.field.setNumber(2, "LcMin", lc / 30)
model.mesh.field.setNumber(2, "LcMax", lc)
model.mesh.field.setNumber(2, "DistMin", 0.15)
model.mesh.field.setNumber(2, "DistMax", 0.5)

model.mesh.field.add("MathEval", 3)
model.mesh.field.setString(3, "F", "Cos(4*3.14*x) * Sin(4*3.14*y) / 10 + 0.101")

model.mesh.field.add("Distance", 4)
model.mesh.field.setNumbers(4, "NodesList", [1])

model.mesh.field.add("MathEval", 5);
model.mesh.field.setString(5, "F", "F4^3 + " + str(lc / 100))

model.mesh.field.add("Box", 6)
model.mesh.field.setNumber(6, "VIn", lc / 15)
model.mesh.field.setNumber(6, "VOut", lc)
model.mesh.field.setNumber(6, "XMin", 0.3)
model.mesh.field.setNumber(6, "XMax", 0.6)
model.mesh.field.setNumber(6, "YMin", 0.3)
model.mesh.field.setNumber(6, "YMax", 0.6)

model.mesh.field.add("Min", 7)
model.mesh.field.setNumbers(7, "FieldsList", [2, 3, 5, 6])

model.mesh.field.setAsBackgroundMesh(7)

factory.synchronize()
model.mesh.generate(2)
gmsh.write("t10.msh")
gmsh.finalize()



# This file reimplements gmsh/tutorial/t6.geo in Python.

import gmsh
import math

model = gmsh.model
factory = model.geo

gmsh.initialize()
gmsh.option.setNumber("General.Terminal", 1)

model.add("t6")

# Copied from t1.py...
lc = 1e-2
factory.addPoint(0, 0, 0, lc, 1)
factory.addPoint(.1, 0,  0, lc, 2)
factory.addPoint(.1, .3, 0, lc, 3)
factory.addPoint(0, .3, 0, lc, 4)
factory.addLine(1, 2, 1)
factory.addLine(3, 2, 2)
factory.addLine(3, 4, 3)
factory.addLine(4, 1, 4)
factory.addCurveLoop([4, 1, -2, 3], 1)
factory.addPlaneSurface([1], 1)
model.addPhysicalGroup(0, [1, 2], 1)
model.addPhysicalGroup(1, [1, 2], 2)
model.addPhysicalGroup(2, [1], 6)
model.setPhysicalName(2, 6, "My surface")
# ...end of copy

# Delete surface 1 and left boundary (curve 4)
factory.remove([[2,1], [1,4]])

# Replace left boundary with 3 new lines
p1 = factory.addPoint(-0.05, 0.05, 0, lc)
p2 = factory.addPoint(-0.05, 0.1, 0, lc)
l1 = factory.addLine(1, p1)
l2 = factory.addLine(p1, p2)
l3 = factory.addLine(p2, 4)

# Recreate surface
factory.addCurveLoop([2, -1, l1, l2, l3, -3], 2)
factory.addPlaneSurface([-2], 1)

# Put 20 points with a refinement toward the extremities on curve 2
factory.mesh.setTransfiniteCurve(2, 20, "Bump", 0.05)

# Put 20 points total on combination of curves l1, l2 and l3 (beware that the
# points p1 and p2 are shared by the curves, so we do not create 6 + 6 + 10 = 22
# points, but 20!)
factory.mesh.setTransfiniteCurve(l1, 6)
factory.mesh.setTransfiniteCurve(l2, 6)
factory.mesh.setTransfiniteCurve(l3, 10)

# Put 30 points following a geometric progression on curve 1 (reversed) and on
# curve 3
factory.mesh.setTransfiniteCurve(1, 30, "Progression", -1.2)
factory.mesh.setTransfiniteCurve(3, 30, "Progression", 1.2)

# Define the Surface as transfinite, by specifying the four corners of the
# transfinite interpolation
factory.mesh.setTransfiniteSurface(1, "Left", [1,2,3,4])

# Recombine the triangles into quads
factory.mesh.setRecombine(2, 1)

# Apply an elliptic smoother to the grid
gmsh.option.setNumber("Mesh.Smoothing", 100)
model.addPhysicalGroup(2, [1], 1)

# When the surface has only 3 or 4 control points, the transfinite constraint
# can be applied automatically (without specifying the corners explictly).
factory.addPoint(0.2, 0.2, 0, 1.0, 7)
factory.addPoint(0.2, 0.1, 0, 1.0, 8)
factory.addPoint(0, 0.3, 0, 1.0, 9)
factory.addPoint(0.25, 0.2, 0, 1.0, 10)
factory.addPoint(0.3, 0.1, 0, 1.0, 11)
factory.addLine(8, 11, 10)
factory.addLine(11, 10, 11)
factory.addLine(10, 7, 12)
factory.addLine(7, 8, 13)
factory.addCurveLoop([13, 10, 11, 12], 14)
factory.addPlaneSurface([14], 15)
for i in range(10,14):
    factory.mesh.setTransfiniteCurve(i, 10)
factory.mesh.setTransfiniteSurface(15)
model.addPhysicalGroup(2, [15], 2)

model.mesh.generate(2)
gmsh.write("t6.msh")
gmsh.finalize()



# This file reimplements gmsh/tutorial/t1.geo in Python. 

# For all the elementary explanations about the general philosphy of entities in
# Gmsh, see the comments in the .geo file. Comments here focus on the specifics
# of the Python API.

# The API is entirely defined in the gmsh module
import gmsh

# Before using any functions in the Python API, Gmsh must be initialized.
gmsh.initialize()

# By default Gmsh will not print out any messages: in order to output messages
# on the terminal, just set the standard Gmsh option "General.Terminal" (same
# format and meaning as in .geo files):
gmsh.option.setNumber("General.Terminal", 1)

# Next we add a new model named "t1" (if gmsh.model.add() is not called a new
# unnamed model will be created on the fly, if necessary):
gmsh.model.add("t1")

# The Python API provides direct access to the internal CAD kernels. The
# built-in CAD kernel was used in t1.geo: the corresponding API functions have
# the "gmsh.model.geo" prefix. To create geometrical points with the built-in
# CAD kernel, one thus uses gmsh.model.geo.addPoint():
#
# - the first 3 arguments are the point coordinates (x, y, z)
#
# - the next (optional) argument is the target mesh size close to the point
#
# - the last (optional) argument is the point tag
lc = 1e-2
gmsh.model.geo.addPoint(0, 0, 0, lc, 1)
gmsh.model.geo.addPoint(.1, 0,  0, lc, 2)
gmsh.model.geo.addPoint(.1, .3, 0, lc, 3)
gmsh.model.geo.addPoint(0, .3, 0, lc, 4)

# The API to create lines with the built-in kernel follows the same
# conventions: the first 2 arguments are point tags, the last (optional one)
# is the line tag.
gmsh.model.geo.addLine(1, 2, 1)
gmsh.model.geo.addLine(3, 2, 2)
gmsh.model.geo.addLine(3, 4, 3)
gmsh.model.geo.addLine(4, 1, 4)

# The philosophy to construct curve loops and surfaces is similar: the first
# argument is now a vector of integers.
gmsh.model.geo.addCurveLoop([4, 1, -2, 3], 1)
gmsh.model.geo.addPlaneSurface([1], 1)

# Physical groups are defined by providing the dimension of the group (0 for
# physical points, 1 for physical curves, 2 for physical surfaces and 3 for
# phsyical volumes) followed by a vector of entity tags. The last (optional)
# argument is the tag of the new group to create.
gmsh.model.addPhysicalGroup(0, [1, 2], 1)
gmsh.model.addPhysicalGroup(1, [1, 2], 2)
gmsh.model.addPhysicalGroup(2, [1], 6)

# Physical names are also defined by providing the dimension and tag of the
# entity.
gmsh.model.setPhysicalName(2, 6, "My surface")

# Before it can be meshed, the internal CAD representation must be synchronized
# with the Gmsh model, which will create the relevant Gmsh data structures. This
# is achieved by the gmsh.model.geo.synchronize() API call for the built-in CAD
# kernel. Synchronizations can be called at any time, but they involve a non
# trivial amount of processing; so while you could synchronize the internal CAD
# data after every CAD command, it is usually better to minimize the number of
# synchronization points.
gmsh.model.geo.synchronize()

# We can then generate a 2D mesh...
gmsh.model.mesh.generate(2)

# ... and save it to disk
gmsh.write("t1.msh")

# This should be called at the end:
gmsh.finalize()



import gmsh
import sys

gmsh.initialize(sys.argv)
gmsh.model.add("square")
gmsh.model.geo.addPoint(0, 0, 0, 0.1, 1)
gmsh.model.geo.addPoint(1, 0, 0, 0.1, 2)
gmsh.model.geo.addPoint(1, 1, 0, 0.1, 3)
gmsh.model.geo.addPoint(0, 1, 0, 0.1, 4)
gmsh.model.geo.addLine(1, 2, 1)
gmsh.model.geo.addLine(2, 3, 2)
gmsh.model.geo.addLine(3, 4, 3)
# try automatic assignement of tag
line4 = gmsh.model.geo.addLine(4, 1)
gmsh.model.geo.addCurveLoop([1, 2, 3, line4], 1)
gmsh.model.geo.addPlaneSurface([1], 6)
gmsh.model.geo.synchronize()
gmsh.model.mesh.generate(2)
gmsh.write("square.msh")



import gmsh
import sys

gmsh.initialize(sys.argv)
gmsh.option.setNumber("General.Terminal", 1)

# Copied from discrete.py...
gmsh.model.add("test");
gmsh.model.addDiscreteEntity(2, 1)
gmsh.model.mesh.setNodes(2, 1, [1, 2, 3, 4],
                         [0., 0., 0.,
                          1., 0., 0.,
                          1., 1., 0.,
                          0., 1., 0.])
gmsh.model.mesh.setElements(2, 1, [2], [[1, 2]],
                            [[1, 2, 3,
                              1, 3, 4]])
# ... end of copy

# Create a new post-processing view
t = gmsh.view.add("some data")

# add 10 steps of model-based data, on the nodes of the mesh
for step in range(0, 10):
    gmsh.view.addModelData(t, step, "test", "NodeData",
                           [1, 2, 3, 4], # tags of nodes
                           [[10.],[10.],[12.+step],[13.+step]]) # data, per node
    
gmsh.view.write(t, "data.msh")
    
gmsh.finalize()



import gmsh
import sys

if len(sys.argv) < 2:
    print "Usage: " + sys.argv[0] + " file [options]"
    exit(0)

gmsh.initialize()
gmsh.option.setNumber("General.Terminal", 1)

gmsh.open(sys.argv[1])

# attempts to run a client selected when opening the file (e.g. a .pro file)
gmsh.onelab.run();

json = gmsh.onelab.get()
print json

gmsh.finalize()



# This file reimplements gmsh/tutorial/t2.geo in Python. Comments focus on the new
# API functions used, compared to the ones introduced in t1.py.

import gmsh
import sys

# nice shortcuts
model = gmsh.model
factory = model.geo

# If sys.argv is passed, Gmsh will parse the commandline in the same way as the
# standalone Gmsh app.
gmsh.initialize(sys.argv)

gmsh.option.setNumber("General.Terminal", 1)

model.add("t2")

# Copied from t1.py...
lc = 1e-2
factory.addPoint(0, 0, 0, lc, 1)
factory.addPoint(.1, 0,  0, lc, 2)
factory.addPoint(.1, .3, 0, lc, 3)
factory.addPoint(0, .3, 0, lc, 4)
factory.addLine(1, 2, 1)
factory.addLine(3, 2, 2)
factory.addLine(3, 4, 3)
factory.addLine(4, 1, 4)
factory.addCurveLoop([4, 1, -2, 3], 1)
factory.addPlaneSurface([1], 1)
model.addPhysicalGroup(0, [1, 2], 1)
model.addPhysicalGroup(1, [1, 2], 2)
model.addPhysicalGroup(2, [1], 6)
model.setPhysicalName(2, 6, "My surface")
# ...end of copy

factory.addPoint(0, .4, 0, lc, 5)
factory.addLine(4, 5, 5)

# Geometrical transformations take a vector of pairs of integers as first
# argument, which contains the list of entities, represented by (dimension, tag)
# pairs. Here we thus translate point 3 (dimension=0, tag=3), by dx=-0.05, dy=0,
# dz=0.
factory.translate([(0, 3)], -0.05, 0, 0)

# The "Duplicata" functionality in .geo files is handled by
# factory.Copy(), which takes a vector of (dim, tag) pairs as input, and
# returns another vector of (dim, tag) pairs.

ov = factory.copy([(0, 3)])
factory.translate(ov, 0, 0.1, 0)

factory.addLine(3, ov[0][1], 7)
factory.addLine(ov[0][1], 5, 8)
factory.addCurveLoop([5,-8,-7,3], 10)
factory.addPlaneSurface([10], 11)

ov = factory.copy([(2, 1), (2, 11)])
factory.translate(ov, 0.12, 0, 0)

print "New surfaces " + str(ov[0][1]) + " and " + str(ov[1][1])

factory.addPoint(0., 0.3, 0.13, lc, 100)
factory.addPoint(0.08, 0.3, 0.1, lc, 101)
factory.addPoint(0.08, 0.4, 0.1, lc, 102)
factory.addPoint(0., 0.4, 0.13, lc, 103)

factory.addLine(4, 100, 110)
factory.addLine(3, 101, 111)
factory.addLine(6, 102, 112)
factory.addLine(5, 103, 113)
factory.addLine(103, 100, 114)
factory.addLine(100, 101, 115)
factory.addLine(101, 102, 116)
factory.addLine(102, 103, 117)

factory.addCurveLoop([115, -111, 3, 110], 118)
factory.addPlaneSurface([118], 119)
factory.addCurveLoop([111, 116, -112, -7], 120)
factory.addPlaneSurface([120], 121)
factory.addCurveLoop([112, 117, -113, -8], 122)
factory.addPlaneSurface([122], 123)
factory.addCurveLoop([114, -110, 5, 113], 124)
factory.addPlaneSurface([124], 125)
factory.addCurveLoop([115, 116, 117, 114], 126)
factory.addPlaneSurface([126], 127)

# The API to create surface loops ("shells") and volumes is similar to the
# one used to create curve loops and surfaces.
factory.addSurfaceLoop([127, 119, 121, 123, 125, 11], 128)
factory.addVolume([128], 129)

# Extrusion works as expected, by providing a vector of (dim, tag) pairs as
# input, the translation vector, and a vector of (dim, tag) pairs as output.
ov2 = factory.extrude([ov[1]], 0, 0, 0.12)

# Mesh sizes associated to geometrical points can be set by passing a vector of
# (dim, tag) pairs for the corresponding points.

factory.mesh.setSize([(0,103), (0,105), (0,109), (0,102), (0,28),
                      (0, 24), (0,6), (0,5)], lc * 3)

model.addPhysicalGroup(3, [129,130], 1)
model.setPhysicalName(3, 1, "The volume")

factory.synchronize()

model.mesh.generate(3)

gmsh.write("t2.msh")

gmsh.finalize()



import gmsh
import math

model = gmsh.model
factory = model.occ

gmsh.initialize()
gmsh.option.setNumber("General.Terminal", 1)

model.add("spline")

for i in range(1, 11):
    factory.addPoint(i, math.sin(i/9.*2.*math.pi), 0, 0.1, i)

factory.addSpline(range(1, 11), 1)
factory.addBSpline(range(1, 11), 2)
factory.addBezier(range(1, 11), 3)

factory.addPoint(0.2,-1.6,0,0.1, 101) 
factory.addPoint(1.2,-1.6,0,0.1, 102) 
factory.addPoint(1.2,-1.1,0,0.1, 103) 
factory.addPoint(0.3,-1.1,0,0.1, 104) 
factory.addPoint(0.7,-1,0,0.1, 105) 

# periodic bspline through the control points
factory.addSpline([103,102,101,104,105,103], 100)

# periodic bspline from given control points and default parameters - will
# create a new vertex
factory.addBSpline([103,102,101,104,105,103], 101)

# general bspline with explicit degree, knots and multiplicities
factory.addPoint(0,-2,0,0.1,201)
factory.addPoint(1,-2,0,0.1,202)
factory.addPoint(1,-3,0,0.1,203)
factory.addPoint(0,-3,0,0.1,204)
factory.addBSpline([201,202,203,204], 200, 2, [], [0,0.5,1], [3,1,3])

factory.synchronize()
gmsh.fltk.run()
gmsh.finalize()



# This file reimplements gmsh/tutorial/t5.geo in Python.

import gmsh
import math

model = gmsh.model
factory = model.geo

gmsh.initialize()
gmsh.option.setNumber("General.Terminal", 1)

model.add("t5")

lcar1 = .1
lcar2 = .0005
lcar3 = .055

factory.addPoint(0.5,0.5,0.5, lcar2, 1)
factory.addPoint(0.5,0.5,0, lcar1, 2)
factory.addPoint(0,0.5,0.5, lcar1, 3)
factory.addPoint(0,0,0.5, lcar1, 4)
factory.addPoint(0.5,0,0.5, lcar1, 5)
factory.addPoint(0.5,0,0, lcar1, 6)
factory.addPoint(0,0.5,0, lcar1, 7)
factory.addPoint(0,1,0, lcar1, 8)
factory.addPoint(1,1,0, lcar1, 9)
factory.addPoint(0,0,1, lcar1, 10)
factory.addPoint(0,1,1, lcar1, 11)
factory.addPoint(1,1,1, lcar1, 12)
factory.addPoint(1,0,1, lcar1, 13)
factory.addPoint(1,0,0, lcar1, 14)

factory.addLine(8,9, 1);   factory.addLine(9,12, 2)
factory.addLine(12,11, 3); factory.addLine(11,8, 4)
factory.addLine(9,14, 5);  factory.addLine(14,13, 6)
factory.addLine(13,12, 7); factory.addLine(11,10, 8)
factory.addLine(10,13, 9); factory.addLine(10,4, 10)
factory.addLine(4,5, 11);  factory.addLine(5,6, 12)
factory.addLine(6,2, 13);  factory.addLine(2,1, 14)
factory.addLine(1,3, 15);  factory.addLine(3,7, 16)
factory.addLine(7,2, 17);  factory.addLine(3,4, 18)
factory.addLine(5,1, 19);  factory.addLine(7,8, 20)
factory.addLine(6,14, 21)

factory.addCurveLoop([-11,-19,-15,-18], 22)
factory.addPlaneSurface([22], 23)
factory.addCurveLoop([16,17,14,15], 24)
factory.addPlaneSurface([24], 25)
factory.addCurveLoop([-17,20,1,5,-21,13], 26)
factory.addPlaneSurface([26], 27)
factory.addCurveLoop([-4,-1,-2,-3], 28)
factory.addPlaneSurface([28], 29)
factory.addCurveLoop([-7,2,-5,-6], 30)
factory.addPlaneSurface([30], 31)
factory.addCurveLoop([6,-9,10,11,12,21], 32)
factory.addPlaneSurface([32], 33)
factory.addCurveLoop([7,3,8,9], 34)
factory.addPlaneSurface([34], 35)
factory.addCurveLoop([-10,18,-16,-20,4,-8], 36)
factory.addPlaneSurface([36], 37)
factory.addCurveLoop([-14,-13,-12,19], 38)
factory.addPlaneSurface([38], 39)

shells = []

# When the tag is not specified, a new one is automatically provided
sl = factory.addSurfaceLoop([35,31,29,37,33,23,39,25,27])
shells.append(sl)

def cheeseHole(x, y, z, r, lc, shells):
    p1 = factory.addPoint(x,  y,  z,   lc)
    p2 = factory.addPoint(x+r,y,  z,   lc)
    p3 = factory.addPoint(x,  y+r,z,   lc)
    p4 = factory.addPoint(x,  y,  z+r, lc)
    p5 = factory.addPoint(x-r,y,  z,   lc)
    p6 = factory.addPoint(x,  y-r,z,   lc)
    p7 = factory.addPoint(x,  y,  z-r, lc)

    c1 = factory.addCircleArc(p2,p1,p7)
    c2 = factory.addCircleArc(p7,p1,p5)
    c3 = factory.addCircleArc(p5,p1,p4)
    c4 = factory.addCircleArc(p4,p1,p2)
    c5 = factory.addCircleArc(p2,p1,p3)
    c6 = factory.addCircleArc(p3,p1,p5)
    c7 = factory.addCircleArc(p5,p1,p6)
    c8 = factory.addCircleArc(p6,p1,p2)
    c9 = factory.addCircleArc(p7,p1,p3)
    c10 = factory.addCircleArc(p3,p1,p4)
    c11 = factory.addCircleArc(p4,p1,p6)
    c12 = factory.addCircleArc(p6,p1,p7)

    l1 = factory.addCurveLoop([c5,c10,c4])
    l2 = factory.addCurveLoop([c9,-c5,c1])
    l3 = factory.addCurveLoop([c12,-c8,-c1])
    l4 = factory.addCurveLoop([c8,-c4,c11])
    l5 = factory.addCurveLoop([-c10,c6,c3])
    l6 = factory.addCurveLoop([-c11,-c3,c7])
    l7 = factory.addCurveLoop([-c2,-c7,-c12])
    l8 = factory.addCurveLoop([-c6,-c9,c2])

    s1 = factory.addSurfaceFilling([l1])
    s2 = factory.addSurfaceFilling([l2])
    s3 = factory.addSurfaceFilling([l3])
    s4 = factory.addSurfaceFilling([l4])
    s5 = factory.addSurfaceFilling([l5])
    s6 = factory.addSurfaceFilling([l6])
    s7 = factory.addSurfaceFilling([l7])
    s8 = factory.addSurfaceFilling([l8])

    sl = factory.addSurfaceLoop([s1, s2, s3, s4, s5, s6, s7, s8])
    v = factory.addVolume([sl])
    shells.append(sl)
    return v

x = 0; y = 0.75; z = 0; r = 0.09
for t in range(1, 6):
    x += 0.166
    z += 0.166
    v = cheeseHole(x, y, z, r, lcar3, shells)
    model.addPhysicalGroup(3, [v], t)

factory.addVolume(shells, 186)

model.addPhysicalGroup(3, [186], 10)
factory.synchronize()

model.mesh.generate(3)
gmsh.write("t5.msh")

gmsh.finalize()



# This files reimplements gmsh/tutorial/t3.geo in Python.

import gmsh
import math

model = gmsh.model
factory = model.geo

gmsh.initialize()
gmsh.option.setNumber("General.Terminal", 1)

model.add("t3")

# Copied from t1.py...
lc = 1e-2
factory.addPoint(0, 0, 0, lc, 1)
factory.addPoint(.1, 0,  0, lc, 2)
factory.addPoint(.1, .3, 0, lc, 3)
factory.addPoint(0, .3, 0, lc, 4)
factory.addLine(1, 2, 1)
factory.addLine(3, 2, 2)
factory.addLine(3, 4, 3)
factory.addLine(4, 1, 4)
factory.addCurveLoop([4, 1, -2, 3], 1)
factory.addPlaneSurface([1], 1)
model.addPhysicalGroup(0, [1, 2], 1)
model.addPhysicalGroup(1, [1, 2], 2)
model.addPhysicalGroup(2, [1], 6)
model.setPhysicalName(2, 6, "My surface")
# ...end of copy

h = 0.1
angle = 90.

# Extruding the mesh in addition to the geometry works as in .geo files: the
# number of elements for each layer and the (end) height of each layer are
# specified in two vectors.
ov = factory.extrude([(2,1)], 0, 0, h, [8,2], [0.5,1])

#/ Rotational and twisted extrusions are available as well with the built-in CAD
# kernel. The last (optional) argument for the Extrude/Revolve/Twist commands
# specified whether the extruded mesh should be recombined or not.
ov = factory.revolve([(2,28)], -0.1,0,0.1, 0,1,0, -math.pi/2, [7])
ov = factory.twist([(2,50)], 0,0.15,0.25, -2*h,0,0, 1,0,0, angle*math.pi/180.,
                   [10], [], True)

model.addPhysicalGroup(3, [1, 2, ov[1][1]], 101)

factory.synchronize()
model.mesh.generate(3)
gmsh.write("t3.msh")
gmsh.finalize()



import gmsh
import sys
gmsh.initialize(sys.argv)
gmsh.option.setNumber("General.Terminal", 1)
gmsh.model.add("square")
gmsh.model.occ.addRectangle(0, 0, 0, 1, 1, 100)
gmsh.model.occ.synchronize()
gmsh.model.mesh.setTransfiniteSurface(100)
gmsh.model.mesh.generate(2)
gmsh.plugin.run("NewView")
print "before get"
type, tags, data, time, numComp = gmsh.view.getModelData(0, 0)
print "after get"
gmsh.finalize()
